=begin
#-------------------------------------------------------------------------------------------------------------------------------------------------
#*************************************************************************************************
# Copyright 2011 Fredo6 - Designed and written August 2011 by Fredo6
#
# Permission to use this software for any purpose and without fee is hereby granted
# Distribution of this software for commercial purpose is subject to:
#  - the expressed, written consent of the author
#  - the inclusion of the present copyright notice in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------
# Name			:  Lib6Plugin.rb
# Original Date	:  10 Dec 2008 - version 3.0
# Type			:  Script library part of the LibFredo6 shared libraries
# Description	:  A utility library about Plugin Configuration for LibFredo6-compliant scripts.
#-------------------------------------------------------------------------------------------------------------------------------------------------
#*************************************************************************************************
=end

module Traductor

#--------------------------------------------------------------------------------------------------------------
#--------------------------------------------------------------------------------------------------------------
# Dialog box for Purge Obsolete files
#--------------------------------------------------------------------------------------------------------------			 
#--------------------------------------------------------------------------------------------------------------			 

class PurgeObsoleteDialog

@@top_dialog = nil

#Invoke the purge Obsolete Dialog
def PurgeObsoleteDialog.invoke
	unique_key = "Traductor_PurgeObsolete_DLG"
	@@top_dialog = PurgeObsoleteDialog.new(unique_key) unless Traductor::Wdlg.check_instance_displayed(unique_key)
end	

#initialization of the dialog box 
def initialize(unique_key)
	MYPLUGIN.load_body(self, __FILE__)
	initialize__(unique_key)
end

end	#class PurgeObsoleteDialog

#--------------------------------------------------------------------------------------------------------------
#--------------------------------------------------------------------------------------------------------------
# Dialog box for Performance and load times
#--------------------------------------------------------------------------------------------------------------			 
#--------------------------------------------------------------------------------------------------------------			 

class PerformanceDialog

@@top_dialog = nil

#Invoke the purge Obsolete Dialog
def PerformanceDialog.invoke
	unique_key = "Traductor_Performance_DLG"
	@@top_dialog = PerformanceDialog.new(unique_key) unless Traductor::Wdlg.check_instance_displayed(unique_key)
end	

#initialization of the dialog box 
def initialize(unique_key)
	MYPLUGIN.load_body(self, __FILE__)
	initialize__(unique_key)
end

end	#class PerformanceDialog

#--------------------------------------------------------------------------------------------------------------
#--------------------------------------------------------------------------------------------------------------
# Dialog box for About Information
#--------------------------------------------------------------------------------------------------------------			 
#--------------------------------------------------------------------------------------------------------------			 

class AboutDialog

@@hsh_dialogs = {}

#Invoke the purge Obsolete Dialog
def AboutDialog.invoke(plugin)
	name = plugin.plugin_name
	unique_key = "Traductor_About_DLG_" + name
	@@hsh_dialogs[name] = AboutDialog.new(unique_key, plugin) unless Traductor::Wdlg.check_instance_displayed(unique_key)
end	

#initialization of the dialog box 
def initialize(unique_key, plugin)
	MYPLUGIN.load_body(self, __FILE__)
	initialize__(unique_key, plugin)
end

end	#Class AboutDialog

#--------------------------------------------------------------------------------------------------------------
#--------------------------------------------------------------------------------------------------------------
# Trace Log files
#--------------------------------------------------------------------------------------------------------------			 
#--------------------------------------------------------------------------------------------------------------			 

class TraceLog

#Compute the list of log files (from the most recent to the oldest)
def TraceLog.list_log_files(flg_all=false)
	nbmax = MYDEFPARAM[:T_DEFAULT_TraceLogNbmax]
	tmpdir = LibFredo6.tmpdir
	current_logfile = LibFredo6.log_file
	@fpat = Regexp.new LibFredo6.rootlog.sub(/\*/, "(\\d+)")
	ls = Dir[File.join(tmpdir, LibFredo6.rootlog)]
	ls.delete current_logfile
	ls.push current_logfile
	ls.reverse!
	ls = ls[0..nbmax] if !flg_all && ls.length > nbmax
	ls
end

#Build a string from the Time of the file
def TraceLog.time_from_filepath(filepath)
	fname = File.basename filepath
	return nil unless fname =~ @fpat
	t = $1.to_f / 1000
	Time.at(t).strftime "%a %d %b %Y - %Hh%M:%S"
end

#Purge the old trace files
def TraceLog.purge
	nbmax = MYDEFPARAM[:T_DEFAULT_TraceLogNbmax]
	ls = list_log_files true
	return 0 if ls.length <= nbmax+10
	ls[nbmax..-1].each { |f| File.delete f }
	ls.length - nbmax
end

end	#class TraceLog

#--------------------------------------------------------------------------------------------------------------
#--------------------------------------------------------------------------------------------------------------
# Dialog box for Trace Log files
#--------------------------------------------------------------------------------------------------------------			 
#--------------------------------------------------------------------------------------------------------------			 

class TraceLogDialog

@@top_dialog = nil

#Invoke the Trace log files Dialog
def TraceLogDialog.invoke
	unique_key = "Traductor_TraceLog_DLG"
	@@top_dialog = TraceLogDialog.new(unique_key) unless Traductor::Wdlg.check_instance_displayed(unique_key)
end	

#initialization of the dialog box 
def initialize(unique_key)
	MYPLUGIN.load_body(self, __FILE__)
	initialize__(unique_key)
end

end	#Class TraceLogDialog

#--------------------------------------------------------------------------------------------------------------
#--------------------------------------------------------------------------------------------------------------
# Debug Log files
#--------------------------------------------------------------------------------------------------------------			 
#--------------------------------------------------------------------------------------------------------------			 

class DebugLog

#Compute the list of log files (from the most recent to the oldest)
def DebugLog.list_debug_files(flg_all=false)
	nbmax = MYDEFPARAM[:T_DEFAULT_DebugLogNbmax]
	tmpdir = LibFredo6.tmpdir
	current_logfile = LibFredo6.debug_file
	@fpat = Regexp.new LibFredo6.rootdebug.sub(/\*/, "(\\d+)")
	ls = Dir[File.join(tmpdir, LibFredo6.rootdebug)]
	ls.delete current_logfile
	ls.push current_logfile
	ls.reverse!
	ls = ls[0..nbmax] if !flg_all && ls.length > nbmax
	ls
end

#Build a string from the Time of the file
def DebugLog.time_from_filepath(filepath)
	return nil unless filepath
	fname = File.basename filepath
	return nil unless fname =~ @fpat
	$1.to_f / 1000
end

#Purge the old trace files
def DebugLog.purge
	nbmax = MYDEFPARAM[:T_DEFAULT_DebugLogNbmax]
	ls = list_debug_files true
	return 0 if ls.length <= nbmax+10
	ls[nbmax..-1].each { |f| File.delete f }
	ls.length - nbmax
end

end	#class DebugLog

#--------------------------------------------------------------------------------------------------------------
#--------------------------------------------------------------------------------------------------------------
# Dialog box for Debug Log files
#--------------------------------------------------------------------------------------------------------------			 
#--------------------------------------------------------------------------------------------------------------			 

class DebugLogDialog

@@top_dialog = nil

#Invoke the Trace log files Dialog
def DebugLogDialog.invoke
	unique_key = "Traductor_DebugLog_DLG"
	@@top_dialog = DebugLogDialog.new(unique_key) unless Traductor::Wdlg.check_instance_displayed(unique_key)
end	

#initialization of the dialog box 
def initialize(unique_key)
	MYPLUGIN.load_body(self, __FILE__)
	initialize__(unique_key)
end

end	#Class DebugLogDialog

end #Module Traductor

