=begin
#-------------------------------------------------------------------------------------------------------------------------------------------------
#*************************************************************************************************
# Copyright 2011 Fredo6 - Designed and written February 2011 by Fredo6

# Permission to use this software for any purpose and without fee is hereby granted
# Distribution of this software for commercial purpose is subject to:
#  - the expressed, written consent of the author
#  - the inclusion of the present copyright notice in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------
# Name			:  Lib6Upgrade.rb
# Original Date	:  23 Feb 2011
# Type			:  Script library part of the LibFredo6 shared libraries
# Description	:  A utility library about Plugin check for upgrade for LibFredo6-compliant scripts.
#-------------------------------------------------------------------------------------------------------------------------------------------------
#*************************************************************************************************
=end


#--------------------------------------------------------------------------------------------------------------
#--------------------------------------------------------------------------------------------------------------
# Class Upgrade: Manage check for Updates
#--------------------------------------------------------------------------------------------------------------
#--------------------------------------------------------------------------------------------------------------

module Traductor

#--------------------------------------------------------------------------------------------------------------
# Class Upgrade: classinitialization part
#--------------------------------------------------------------------------------------------------------------			 

#Class variables initialization
unless defined?(self::Upgrade)
class Upgrade
	@@top_dialog = nil
	@@only_registered = true
	@@time_last_check = nil
	@@external_check = false
end
end

class Upgrade

@@tmpdir = (RUN_ON_MAC) ? "/tmp" : ENV["TEMP"].gsub(/\\/, '/')

#--------------------------------------------------------------------------------------------------------------
# Class methods: Manage top level actions
#--------------------------------------------------------------------------------------------------------------

#Show the dialog box for Check for update.
# An optional list of plugin names can be passed
def Upgrade.top_dialog(*hoptions)
	hsh = {}
	hoptions.each { |h| h.each { |key, val| hsh[key] = val } }
	unique_key = "Traductor_Upgrade_DLG"
	@@top_dialog = Upgrade.new(unique_key, hsh) unless Traductor::Wdlg.check_instance_displayed(unique_key)
end

#Show the dialog box for Check for update.
# An optional list of plugin names can be passed
def Upgrade.time_for_check?
	tcheck = Upgrade.date_next_check
	if @@top_dialog == nil && tcheck < Time.now.to_f
		UpgradeWarning.show
		Upgrade.date_next_check Time.now.to_f + Upgrade.duration_next_check * 86400
	end	
end

#Verify if the current date will trigger a check for update
def Upgrade.date_next_check(time=nil)
	regkey = "Time_of_Next_Check"
	
	#Writing duration as number of seconds
	if time
		Sketchup.write_default "LibFredo6", regkey, time.to_s
		
	#Getting duration in nb days	
	else
		time = Sketchup.read_default "LibFredo6", regkey
		time = Traductor.string_to_float_formula(time)
		time = 0 unless time
	end	
	time
end

#Get or set the duration until next check
def Upgrade.duration_next_check(duration=nil)
	regkey = "Duration_of_Next_Check"
	
	#Writing duration in days
	if duration
		duration = 999 if duration > 999
		duration = 0 if duration < 0
		duration = Traductor.string_to_float_formula duration.to_s
		Sketchup.write_default "LibFredo6", regkey, duration.to_s if duration
		
	#Getting duration in nb days	
	else
		duration = Sketchup.read_default "LibFredo6", regkey
		duration = MYDEFPARAM[:T_DEFAULT_DurationNextCheck] unless duration && duration.length > 0
		duration = Traductor.string_to_float_formula duration.to_s
		if !duration
			duration = 15
			Sketchup.write_default "LibFredo6", regkey, duration.to_s
		elsif duration > 999
			duration = 999
			Sketchup.write_default "LibFredo6", regkey, duration.to_s
		end	
	end	
	(duration && duration >= 0) ? duration : 0
end

#Get or Set the last time of Check for Update
def Upgrade.time_last_check(time=nil)
	regkey = "Time_of_Last_Check"
	
	#Writing duration as number of seconds
	if time
		Sketchup.write_default "LibFredo6", regkey, time.to_s
		
	#Getting duration in nb days	
	else
		time = Sketchup.read_default "LibFredo6", regkey
		time = Traductor.string_to_float_formula(time)
		time = 0 unless time
	end	
	time
end

#-------------------------------------------------------------------------------------------
# Instance Methods for Upgrade class
#-------------------------------------------------------------------------------------------

#initialization of the Upgrade environment (single instance)
def initialize(unique_key, hoptions)	
	MYPLUGIN.load_body(self, __FILE__)
	initialize__(unique_key, hoptions)	
end

end	#class Upgrade

#----------------------------------------------------------------------------------------------------------
# Class UpgradeWarning: Warning dialog box 
#----------------------------------------------------------------------------------------------------------

class UpgradeWarning

@@warning_dialog = nil

def UpgradeWarning.show
	unique_key = "Traductor_Upgrade_DLG_WARNING"
	@@warning_dialog = UpgradeWarning.new(unique_key) #unless @@warning_dialog
	nil
end

#Create the warning dialog box
def initialize(unique_key)
	MYPLUGIN.load_body(self, __FILE__)
	initialize__(unique_key)
end	

end	#class UpgradeWarning

end #Module Traductor

