=begin
#-------------------------------------------------------------------------------------------------------------------------------------------------
#*************************************************************************************************
# Designed Dec. 2008 by Fredo6

# Permission to use this software for any purpose and without fee is hereby granted
# Distribution of this software for commercial purpose is subject to:
#  - the expressed, written consent of the author
#  - the inclusion of the present copyright notice in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------
# Name			:  Lib6Web.rb
# Original Date	:  10 Dec 2008 - version 3.0
# Type			:  Script library part of the LibFredo6 shared libraries
# Description	:  A utility library to assist web dialog design.
#-------------------------------------------------------------------------------------------------------------------------------------------------
#*************************************************************************************************
=end

module Traductor

#--------------------------------------------------------------------------------------------------------------
# Class Wdlg: Manage web dialogs
#--------------------------------------------------------------------------------------------------------------			 

class Wdlg

@@hsh_unique = {}

#Check if the web dialog box has already an instance visible, based on the unique key
#Return the instance if so or nil
def Wdlg.check_instance_displayed(key, bring_to_front=true)
	return nil unless key
	wdlg = @@hsh_unique[key]
	wdlg.bring_to_front if wdlg && bring_to_front
	wdlg
end

#Retrieve browser information
def Wdlg.browser_info()
	param = Registry.browser_info_load 
	unless param
		view = Sketchup.active_model.active_view
		browser = (RUN_ON_MAC) ? "Safari" : "IE7"
		return [browser, view.vpwidth, view.vpheight]
	end
	browser, sw, sh = param
	[browser, sw.to_i, sh.to_i]	
end

def initialize(title, hkey=nil, resizable=true, scrollable=false)
	Traductor.use_libraries
	MYPLUGIN.load_body(self, "Lib6WebJS.rb")
	MYPLUGIN.load_body(self, __FILE__)
	initialize__(title, hkey, resizable, scrollable)
end

end	#class Wdlg

#--------------------------------------------------------------------------------------------------------------
# Class WMsgBox: Utlities to manage Message boxbased on Web Dialog
#--------------------------------------------------------------------------------------------------------------			 

class WMsgBox

#Invoke a modal message box with all possibilities
def WMsgBox.call(message, type=nil, title=nil, lbuttons=["OK"], callback=nil, context=nil)
	wm = WMsgBox.new message, type, title, lbuttons, callback, context
	wm.invoke
end

#Standard confirmation box for changes
#Return -1 to ignore changes, 0 to cancel and go back, +1 to save changes
def WMsgBox.confirm_changes(txtchanges=nil)
	html_text = T6[:T_WARNING_ActiveChange]
	html_text += "\n<b><span style='color: red ; text-align: center'>" + txtchanges + "</span></b>" if txtchanges
	html_text += "\n" + T6[:T_WARNING_WhatToDo]	
	lbuttons = [:T_BUTTON_Ignore, :T_BUTTON_DecideLater, '^', :T_BUTTON_GoBack, '~', :T_BUTTON_Save]
	title = T6[:T_STR_ConfirmChange]
	UI.beep
	status = WMsgBox.call html_text, nil, title,  T6[lbuttons]
	return ['I', 'L', 'B', 'S'][status]
end

#Create the class instance
def initialize(message, type=nil, title=nil, lbuttons=["OK"], callback=nil, context=nil)
	MYPLUGIN.load_body(self, __FILE__)
	initialize__(message, type, title, lbuttons, callback, context)
end

end	#class WMsgBox

#--------------------------------------------------------------------------------------------------------------
# Class SysInfo: Utlities to get some information about the system
#--------------------------------------------------------------------------------------------------------------			 

class SysInfo
@@hsh_prop = nil
@@asynchronous = (RUN_ON_MAC) ? true : false	#For Mac asynchronous bug


def SysInfo.[](prop)
	return nil unless prop && prop.strip != ""
	SysInfo.init unless @@hsh_prop
	(@@hsh_prop) ? @@hsh_prop[prop.strip.upcase] : nil
end
	
def SysInfo.init
	return if @@hsh_prop
	text = ""
	text += T_HTML_DOC_TYPE + '<HTML><HEAD>' + T_HTML_SCRIPT_TYPE
	
	txt = %Q~
	function getinfo() {
		Transfer ("UserAgent", navigator.userAgent) ;
		Transfer ("Width", screen.width) ;
		Transfer ("Height", screen.height) ;
		Transfer ("AvailWidth", screen.availWidth) ;
		Transfer ("AvailHeight", screen.availHeight) ;
		Transfer ("AvailTop", screen.availTop) ;
		Transfer ("AvailLeft", screen.availLeft) ;
		Transfer ("ScreenLeft", window.screenTop) ;
		Transfer ("ScreenLeft", window.screenLeft) ;
	}	
	function Transfer(prop, val) {
		msg = '!!' + prop + ';;;;' + val
		obj = document.getElementById ('HH_SCRIPT_HH') ;
		obj.value += msg
		window.location = 'skp:SysInfo@' + msg ;
	}
	~
	
	text += "<SCRIPT>#{txt}</SCRIPT>"	
	text += "</HEAD><BODY onload='getinfo(); '>"
	text += "<input id='HH_SCRIPT_HH' type='hidden' value=''></input>"
	text += "</BODY></HTML>"
	
	unless @@hsh_prop
		@suwdlg = UI::WebDialog.new
		@suwdlg.set_size 10, 10
		@suwdlg.set_html text
		@suwdlg.add_action_callback("SysInfo") { |d, p| SysInfo.callback p }	
		if RUN_ON_MAC
			@suwdlg.show 
			@suwdlg.close
		else
			@suwdlg.show_modal {@suwdlg.close}
		end	
	end	
end

def SysInfo.callback(param)
	if @@asynchronous
		param = @suwdlg.get_element_value "HH_SCRIPT_HH"
	end
	lsv = param.split '!!'
	lsv.each do |p|
		next if p == ""
		SysInfo.store p
	end	
end

def SysInfo.store(p)
	ls = p.split(';;;;')
	code = ls[0]
	val = ls[1]
	
	begin
		case code
		when /userAgent/i
			key = "BROWSER"
			if val =~ /MSIE\s(\d)/i
				value = 'IE' + $1
			elsif val =~ /MSIE/i
				value = 'IE'
			elsif val =~ /Safari/i
				value = (val =~ /version/i) ? value = 'Safari3' : 'Safari2'
			elsif val =~ /Mac/i
				value = 'Safari3'
			else
				value = ''
			end
		else
			key = code
			value = val.to_i
		end
	rescue
		return
	end
	
	@@hsh_prop = {} unless @@hsh_prop
	@@hsh_prop[key] = value
end

end	#class SysInfo

end #Module Traductor
