=begin
#-------------------------------------------------------------------------------------------------------------------------------------------------
#*************************************************************************************************
# Designed July 2013 by Fredo6

# Permission to use this software for any purpose and without fee is hereby granted
# Distribution of this software for commercial purpose is subject to:
#  - the expressed, written consent of the author
#  - the inclusion of the present copyright notice in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------
# Name			:   Lib6DirectionManager.rb
# Original Date	:   31 Jul 2013
# Description	:   Manage interactive selection of directions (plane or vector)
#-------------------------------------------------------------------------------------------------------------------------------------------------
#*************************************************************************************************
=end

module Traductor

#------------------------------------------------------------------
# Text for Axes and Planes
#------------------------------------------------------------------

T6[:T_TIP_PlaneLocal] = "LOCAL: Plane is based on Local axes for Components or Groups"	
T6[:T_TIP_Plane_None] = "No privileged plane"
T6[:T_TIP_Plane_X] = "Vertical Plane YZ [Red]"
T6[:T_TIP_Plane_Y] = "Vertical Plane XZ [Green]"
T6[:T_TIP_Plane_Z] = "Horizontal Plane XY [Blue]"
T6[:T_TIP_Plane_Custom] = "Custom Plane"
T6[:T_TIP_Plane_Ask] = "Specify interactively a Custom Plane"

T6[:T_TIP_VectorLocal] = "LOCAL: Direction is based on Local axes for Components or Groups"	
T6[:T_TIP_Vector_None] = "No privileged direction"
T6[:T_TIP_Vector_X] = "Axis X [Red]"
T6[:T_TIP_Vector_Y] = "Axis Y [Green]"
T6[:T_TIP_Vector_Z] = "Vertical Axis [Blue]"
T6[:T_TIP_Vector_Custom] = "Custom Direction"
T6[:T_TIP_Vector_Ask] = "Specify interactively a Custom Direction"

T6[:TIP_VectorPickPlaneEdge] = "Pick a Plane (normal) or an Edge (direction)"
T6[:TIP_VectorCancel] = "ESC to cancel and exit Direction Picker"
T6[:TIP_PlanePickPlaneEdge] = "Pick a Plane or an Edge (perpendicular plane)"
T6[:TIP_PlaneCancel] = "ESC to cancel and exit Plane Picker"

T6[:TIP_PlaneTitle] = "Custom Plane Picker"
T6[:TIP_VectorTitle] = "Custom Direction Picker"

#=============================================================================================
#=============================================================================================
# Class DirectionManager: main class for the Direction Manager Interactive tool
#=============================================================================================
#=============================================================================================

class DirectionManager

#-----------------------------------------------------------
# DIRECTION: Instance initialization
#-----------------------------------------------------------

#INIT: Class initialization
def initialize__(*hargs)
	#Initialization
	@model = Sketchup.active_model
	@selection = @model.selection
	@view = @model.active_view
	@ph = @view.pick_helper	
	@rendering_options = Sketchup.active_model.rendering_options
	@tr_id = Geom::Transformation.new
	@viewtracker = G6::ViewTracker.new
	@button_down = false
	@dragging = false
	@pixel_dragging = 5
	@ip = Sketchup::InputPoint.new
	@shape_grid = G6::ShapeGrid.new 4
	
	@code = :no
	@scope_local = true
	
	#Parsing the arguments
	hargs.each do |harg|	
		harg.each { |key, value|  parse_args(key, value) } if harg.class == Hash
	end
	
	#Initialization
	text_init
	colors_init
	cursor_init
	
	@state = :origin
	@id_cursor = 0
end

#INIT: Parse the parameters of the Face Picker
def parse_args(key, value)
	skey = key.to_s
	case skey
	when /notify_proc/i
		@notify_proc = value
	when /code/i
		@code = value
		compute_direction_code
	when /scope_local/i
		@scope_local = value
	when /vec_dir_custom/i
		@vec_dir_custom = Geom::Vector3d.new(*value) if value.class == Array
	when /select_vector/i
		@select_vector = value
	end	
end	

#INIT: Initialize texts
def text_init
	if @select_vector
		@txt_ask = T6[:T_TIP_Vector_Ask]
		@txt_local = T6[:T_TIP_VectorLocal]
		@txt_z = T6[:T_TIP_Vector_Z]
		@txt_x = T6[:T_TIP_Vector_X]
		@txt_y = T6[:T_TIP_Vector_Y]
		@txt_c = T6[:T_TIP_Vector_Custom]
		@txt_no = T6[:T_TIP_Vector_None]
	else
		@txt_ask = T6[:T_TIP_Plane_Ask]
		@txt_local = T6[:T_TIP_PlaneLocal]
		@txt_z = T6[:T_TIP_Plane_Z]
		@txt_x = T6[:T_TIP_Plane_X]
		@txt_y = T6[:T_TIP_Plane_Y]
		@txt_c = T6[:T_TIP_Plane_Custom]
		@txt_no = T6[:T_TIP_Plane_None]
	end
	
	@tip_select_plane = T6[:TIP_PlanePickPlaneEdge] + "\n" + T6[:TIP_PlaneCancel]
	@tip_select_vector = T6[:TIP_VectorPickPlaneEdge] + "\n" + T6[:TIP_VectorCancel]
	
	#Defining the title box
	@title = (@select_vector) ? T6[:TIP_VectorTitle] : T6[:TIP_PlaneTitle]
	wtext, htext = G6.simple_text_size(@title)
	dec = 80
	x = (@view.vpwidth - wtext - dec) * 0.5
	y = 0
	h = 2 * htext
	w = wtext + dec
	@pt_title = Geom::Point3d.new(x + dec * 0.5, htext * 0.5)
	@box_title = [[x,y], [x+w, y], [x+w, y+h], [x, y+h]].collect { |a, b| Geom::Point3d.new a, b }
	@color_bk_title = 'yellow'
	@color_fr_title = 'purple'
end

def cursor_init
	@id_cursor_arrow_exit = Traductor.create_cursor "Cursor_Arrow_Exit", 0, 0	
	@id_cursor_arrow_face = Traductor.create_cursor "Cursor_Arrow_Face", 0, 0	
	@id_cursor = @id_cursor_arrow_face
end

#INIT: Initialize colors
def colors_init	
	@hsh_boxinfo_vector = { :bk_color => 'lightblue', :fr_color => 'green' }					 
	@hsh_boxinfo_plane = { :bk_color => 'lightgreen', :fr_color => 'blue' }					 
end

#INIT: Return the text related to the plane
def info_text
	case @code
	when :x
		@txt_x
	when :y
		@txt_y
	when :z
		@txt_z
	when :c
		@txt_c
	when :no
		@txt_no
	end	
end

#-----------------------------------------------------------
# DIRECTION: Manage the getting and setting of directions
#-----------------------------------------------------------

def get_direction_code ; @code ; end
def get_vector ; @vec_dir ; end
def get_vector_custom ; @vec_dir_custom ; end
def get_scope_local ; @scope_local ; end

#Set the direction code and notify the caller
def set_direction_code(code, toggle=false)
	@code = code
	@scope_local = !@scope_local if toggle
	compute_direction_code
	@notify_proc.call @vec_dir, @scope_local if @notify_proc
end

#Compute the direction based on the code
def compute_direction_code
	case @code
	when :x
		@vec_dir = X_AXIS
	when :y
		@vec_dir = Y_AXIS
	when :z
		@vec_dir = Z_AXIS
	when :c
		@vec_dir = @vec_dir_custom
	when :no
		@vec_dir = nil
	end
end

#Toggle the scope of directions
def toggle_direction_scope
	@scope_local = !@scope_local
	@notify_proc.call @vec_dir, @scope_local if @notify_proc
end	

#Enter the private mode of custom selection
def ask_mode? ; @ask_mode ; end
def enter_ask_mode
	@ask_mode = !@ask_mode
	@view.invalidate
end

#UNDO: Cancel and undo methods
def onCancel(flag, view)
	return false if !@ask_mode || flag != 0
	@ask_mode = false
	true
end

def onSetCursor
	return false unless @ask_mode
	
	if @picked_face
		id_cursor = @id_cursor_arrow_face
	else	
		id_cursor = @id_cursor_arrow_exit
	end	
	UI.set_cursor id_cursor
	true
end

#-------------------------------------------
# Picking and hovering Faces and Edges
#-------------------------------------------

#PICK: Manage actions on a mouse move
def onMouseMove(flags, x, y, view)
	return false unless @ask_mode
	@xmove = x
	@ymove = y
	@vec_dir = nil
	
	@picked_face, @picked_edge, @tr, @parent = hover_under_mouse(flags, x, y, view)
	@ip.pick view, x, y
	if @picked_edge
		pt1 = @tr * @picked_edge.start.position
		pt2 = @tr * @picked_edge.end.position
		@vec_dir = G6.transform_vector pt1.vector_to(pt2), @tr
		if @select_vector
			d1 = @ip.position.distance pt1
			d2 = @ip.position.distance pt2
			r = d1 / (d1 + d2)
			@origin = Geom.linear_combination 1-r, pt1, r, pt2
		else	
			@xdir = @vec_dir.axes[0]
			@edge_pts = [pt1, pt2]
		end	
	elsif @picked_face
		@vec_dir = G6.transform_vector @picked_face.normal, @tr
		if @select_vector
			plane = [@tr * @picked_face.vertices[0].position, @vec_dir]
			@origin = Geom.intersect_line_plane view.pickray(x, y), plane
		else
			@xdir = @vec_dir.axes[0]
			@face_pts = @picked_face.outer_loop.vertices.collect { |vx| @tr * vx.position }
		end	
	end	
		
	view.invalidate
	onSetCursor
	
	true
end

#HOVER: Identify which objects (face and edge)  are under the mouse
def hover_under_mouse(flags, x, y, view)
	precision = 0
	@ph.do_pick x, y, precision
	
	ph_face = @ph.picked_face
	ph_edge = @ph.picked_edge
	elt = [ph_edge, ph_face].find { |a| a }

	#Finding the parent and transformation
	tr = @tr_id
	parent = nil
	return nil unless elt
	for i in 0..@ph.count
		ls = @ph.path_at(i)
		if ls && ls.include?(elt)
			parent = ls[-2]
			tr = @ph.transformation_at(i)
			break
		end
	end	
	tr = @tr_id unless tr
	parent = @model unless parent
	[ph_face, ph_edge, tr, parent]
end

#---------------------------------------------------------------------------------------------
# DRAW: Methods to manage drawing
#---------------------------------------------------------------------------------------------

#DRAW: Top method for drawing in the viewport
def draw(view)
	return false unless @ask_mode
	
	draw_component(view, @parent) if @parent && @parent != @model
	
	#Draw the title
	draw_title view
	
	#Draw the directions
	if @select_vector
		draw_in_vector_mode view
	else
		draw_in_plane_mode view
	end	
end

#DRAW: Draw the title box
def draw_title(view)
	view.drawing_color = @color_bk_title
	view.draw2d GL_POLYGON, @box_title
	view.line_stipple = ''
	view.line_width = 3
	view.drawing_color = @color_fr_title
	view.draw2d GL_LINE_LOOP, @box_title
	view.draw_text @pt_title, @title
end

#DRAW: Draw for Plane mode
def draw_in_plane_mode(view)	
	#Highlighting the picked face
	if @picked_edge
		view.line_stipple = ''
		view.line_width = 3
		view.drawing_color = 'orange'
		view.draw GL_LINE_STRIP, @edge_pts.collect { |pt| G6.small_offset view, pt }
		@shape_grid.draw(view, G6.small_offset(view, @ip.position), @vec_dir, @xdir, 15)
	
	#Highlighting the picked face
	elsif @picked_face
		view.line_stipple = ''
		view.line_width = 3
		view.drawing_color = Traductor::Couleur.color_at_face('red', @picked_face)
		view.draw GL_LINE_LOOP, @face_pts.collect { |pt| G6.small_offset view, pt }
		@shape_grid.draw(view, G6.small_offset(view, @ip.position), @vec_dir, @xdir, 15)
	end
	
	#Draw the tooltip for help
	G6.draw_rectangle_multi_text view, @xmove, @ymove, @tip_select_plane, @hsh_boxinfo_plane
	
	true
end

#DRAW: Draw for Vector mode
def draw_in_vector_mode(view)	
	#Highlighting the picked direction on edge or face
	if @picked_edge || @picked_face
		color = 'red'
		color = Traductor::Couleur.color_at_face(color, @picked_face) if @picked_face
		view.line_stipple = ''
		view.line_width = 4
		view.drawing_color = color
		size = view.pixels_to_model 150, @origin
		ptbeg = @origin.offset(@vec_dir, -size)
		ptend = @origin.offset(@vec_dir, size) 
		view.draw GL_LINE_STRIP, [ptbeg, ptend]
		
		vec = ptbeg.vector_to ptend
		axes = vec.axes
		vecy = axes[1]
		size = view.pixels_to_model 10, ptbeg
		ptmid = ptbeg.offset vec, -size
		pt1 = ptbeg.offset vecy, size
		pt2 = ptbeg.offset vecy, -size
		view.draw GL_POLYGON, [pt1, pt2, ptmid]
		ptmid = ptend.offset vec, size
		pt1 = ptend.offset vecy, size
		pt2 = ptend.offset vecy, -size
		view.draw GL_POLYGON, [pt1, pt2, ptmid]
	end
	
	#Draw the tooltip for help
	G6.draw_rectangle_multi_text view, @xmove, @ymove, @tip_select_vector, @hsh_boxinfo_vector
	
	true
end

#DRAW: draw the boundary box of a component
def draw_component(view, parent)
	return unless parent.valid?
	view.line_stipple = ''
	view.line_width = 1
	view.drawing_color = 'gray'
	llines = G6.grouponent_box_lines(view, parent, @tr, 10)
	view.draw GL_LINES, llines unless llines.empty?
end

#---------------------------------------------------------------------------------------------
# KEYBOARD: Keyboard management
#---------------------------------------------------------------------------------------------

def handle_key_down(key)
	false
end

def handle_key_up(key)
	false
end

#---------------------------------------------------------------------------------------------
# CLICK: Click management
#---------------------------------------------------------------------------------------------

#CLICK: Button click DOWN
def onLButtonDown(flags, x, y, view)
	return false unless @ask_mode
	@button_down = true
	@xdown = x
	@ydown = y
	true
end

#CLICK: Button click UP
def onLButtonUp(flags, x, y, view)
	return false unless @ask_mode
	@button_down = false
	onMouseMove(flags, x, y, view)
	validate_direction
	true
end

#CLICK: Double Click received
def onLButtonDoubleClick(flags, x, y, view)
	return false unless @ask_mode
	
end

#CLICK: Validate the direction and set it
def validate_direction
	return unless @vec_dir
	@ask_mode = false
	@vec_dir_custom = @vec_dir
	UI.start_timer(0) { set_direction_code :c }
end

#------------------------------------------------------------------------------------
# MENU: Contribution to Contextual Menu
#------------------------------------------------------------------------------------

#Contextual menu
def contextual_menu_contribution(cxmenu)
	cxmenu.add_sepa
	mnu_navig_left = Traductor.encode_menu @tip_navig_left
	mnu_navig_right = Traductor.encode_menu @tip_navig_right	
	mnu_navig_down = Traductor.encode_menu @tip_navig_down
	mnu_navig_up = Traductor.encode_menu @tip_navig_up
	cxmenu.add_item(mnu_navig_left) { history_undo } unless history_proc_gray(:undo)
	cxmenu.add_item(mnu_navig_right) { history_redo } unless history_proc_gray(:redo)
	cxmenu.add_item(mnu_navig_down) { history_clear } unless history_proc_gray(:clear)
	cxmenu.add_item(mnu_navig_up) { history_restore } unless history_proc_gray(:restore)
end

#------------------------------------------------------------------------------------
# PALETTE: Palette contribution
#------------------------------------------------------------------------------------

#PALETTE: palette contribution from the Face Picker
def palette_contribution(palette, *hargs)
	@draw_local = self.method "draw_button_opengl"

	#Button for Local or Model
	vproc = proc { @scope_local }
	hsh = { :value_proc => vproc, :height => 32, :width => 32, :tooltip => @txt_local, 
	        :draw_proc => [:std_group, @draw_local], :main_color => 'gray' }
	palette.declare_button(:pal_plane_scope, hsh) { toggle_direction_scope }

	#Buttons for Plane
	wid = 20
	hshb = { :height => 16, :width => wid, :draw_proc => @draw_local }
	
	lgproc = proc { set_direction_code :x, true }
	vproc = proc { @code == :x }
	hsh = { :value_proc => vproc, :long_click_proc => lgproc, :tooltip => @txt_x, :text => 'X', :rank => 1 }
	palette.declare_button(:pal_plane_X, hsh, hshb) { set_direction_code :x }

	vproc = proc { @code == :no }
	hsh = { :value_proc => vproc, :tooltip => @txt_no }
	palette.declare_button(:pal_plane_N, hsh, hshb) { set_direction_code :no }

	lgproc = proc { set_direction_code :y, true }
	vproc = proc { @code == :y }
	hsh = { :value_proc => vproc, :long_click_proc => lgproc, :tooltip => @txt_y, :text => 'Y', :rank => 1 }
	palette.declare_button(:pal_plane_Y, hsh, hshb) { set_direction_code :y }

	grayed_proc = proc { !@vec_dir_custom }
	lgproc = proc { set_direction_code :c, true }
	vproc = proc { @code == :c }
	hsh = { :value_proc => vproc, :long_click_proc => lgproc, :grayed_proc => grayed_proc, :tooltip => @txt_c, :text => 'C' }
	palette.declare_button(:pal_plane_C, hsh, hshb) { set_direction_code :c }

	lgproc = proc { set_direction_code :z, true }
	vproc = proc { @code == :z }
	hsh = { :value_proc => vproc, :long_click_proc => lgproc, :tooltip => @txt_z, :text => 'Z', :rank => 1 }
	palette.declare_button(:pal_plane_Z, hsh, hshb) { set_direction_code :z }

	hsh = { :tooltip => @txt_ask, :text => '?' }
	palette.declare_button(:pal_plane_A, hsh, hshb) { enter_ask_mode }
end

#PALETTE: Custom drawing of buttons
def draw_button_opengl(symb, dx, dy, main_color, frame_color, selected, grayed)
	code = symb.to_s
	lst_gl = []
	dx2 = dx / 2
	dy2 = dy / 2
	color = (grayed) ? 'gray' : frame_color
	
	case code
	
	when /pal_plane_scope/
		dx3 =dx/3
		origin = Geom::Point3d.new(dx3+2, 6)
		pts = [origin, Geom::Point3d.new(dx3+2, dy-6)]
		lst_gl.push [GL_LINE_STRIP, pts, 'blue', 1, '']		
		pts = [origin, Geom::Point3d.new(dx3+10, 6)]
		lst_gl.push [GL_LINE_STRIP, pts, 'red', 1, '']		
		pts = [origin, Geom::Point3d.new(dx2+6, dy2)]
		lst_gl.push [GL_LINE_STRIP, pts, 'green', 1, '']		
		
	when /pal_plane_(.)/
		dec = 0
		pts = []
		pts.push Geom::Point3d.new(dec, dec)
		pts.push Geom::Point3d.new(dx-dec, dec)
		pts.push Geom::Point3d.new(dx-dec, dy-dec)
		pts.push Geom::Point3d.new(dec, dy-dec)
		case $1
		when 'X'
			color = 'tomato'
		when 'Y'
			color = 'limegreen'
		when 'Z'
			color = 'skyblue'
		when 'N'
			color = nil
		when 'C', 'A'
			color = (grayed) ? 'lightgrey' : 'yellow'
		end	
		lst_gl.push [GL_POLYGON, pts, color] if color
		if color && selected
			lst_gl.push [GL_LINE_LOOP, pts, 'white', 2, '']
		elsif !color
			color = (selected) ? 'red' : 'black'
			lst_gl.push [GL_LINE_LOOP, pts, color, 1, '-']
			lst_gl.push [GL_LINE_STRIP, [pts[0], pts[2]], color, 1, '-']
			lst_gl.push [GL_LINE_STRIP, [pts[1], pts[3]], color, 1, '-']
		end	
				
	end	#case code
	
	lst_gl
end

end	#class DirectionManager

end	#End Module Traductor
